// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted 
// by Texas Instruments is distributed as "freeware".  You may, 
// only under TI's copyright in the Program, use and modify the 
// Program without any charge or restriction.  You may 
// distribute to third parties, provided that you transfer a 
// copy of this license to the third party and the third party 
// agrees to these terms by its first use of the Program. You 
// must reproduce the copyright notice and any other legend of 
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains 
// copyrighted material, trade secrets and other TI proprietary 
// information and is protected by copyright laws, 
// international copyright treaties, and trade secret laws, as 
// well as other intellectual property laws.  To protect TI's 
// rights in the Program, you agree not to decompile, reverse 
// engineer, disassemble or otherwise translate any object code 
// versions of the Program to a human-readable form.  You agree 
// that in no event will you alter, remove or destroy any 
// copyright notice included in the Program.  TI reserves all 
// rights not specifically granted under this license. Except 
// as specifically provided herein, nothing in this agreement 
// shall be construed as conferring by implication, estoppel, 
// or otherwise, upon you, any license or other right under any 
// TI patents, copyrights or trade secrets.
//
//*******************************************************************************
//  ScanIF Demo - MSP430FW42x Daughter Card w/ 2 LC sensors
//
//  Description:	Detect rotation of a disc using the Scan IF module of the
//					MSP430FW42x. The daughter card is compatible with MSP430
//					target board MSP-TS430PM64.
//
//
//
//                MSP430FW42x
//             -----------------
//         /|\|              XIN|-  
//          | |                 | 32,768 Hz
//          --|RST          XOUT|-
//            |                 |
//            |             P6.4|-->LED 1
//      LC1-->|SIFCH0           |
//            |             P6.5|-->LED 2
//      LC2-->|SIFCH1           |
//            |             P6.6|-->LED 3
//            |                 |
//            |             P6.7|-->LED 4
//
//  Zack Mak
//  Texas Instruments, Inc
//  January 2014
//******************************************************************************

#include "msp430xw42x.h"
#define __msp430xW42x

#include "LCD.h"
#include "ScanIF.h"

char calibrating = 0;

void main(void)
{
	WDTCTL = WDTPW + WDTHOLD;		// disable Watchdog
	FLL_CTL0 |= XCAP10PF;			// Configure load caps

	P1DIR=0xff;
	P1OUT=0;
	P2DIR=0xff;
	P2OUT=0;
	P3DIR=0xff;
	P3OUT=0;
	P4DIR=0xff;
	P4OUT=0;
	P5DIR=0xff;
	P5OUT=0;
	P6DIR=0xfc;
	P6OUT=0;

	P6SEL |= 0x03;					// Bit 0-1 = ScanIF input channel
	P6OUT &= ~(BIT4 + BIT5);		// Turn off LEDs (if connected)
	
	InitLCD();
	DisplayCAL();
	_EINT();
	
	InitScanIF();					// Initialization of ScanIf module
	
	SIFCTL4 |= SIFCNTRST;			// Enable reset SIFCNT
	DisplayByte(SIFCNT);			// Display the number on the LCD. Also reading of SIFCNT resets itself
	SIFCTL4 &= ~SIFCNTRST;			// Disable reset SIFCNT

	while (1)
	{
#if 1
		// The system sleeps for most of the time until
		// the ScanIF interrupt wakes up the system for calibration.
		calibrating = 0;
		LPM3;
		calibrating = 1;
		P6OUT |= BIT4;			// Turn on LED
		CalScanIF();
		P6OUT &= ~BIT4;			// Turn off LED
#else
		// This section of code shows the state of SIFxOUT using LED output
		if (SIFCTL3 & 0x01)		// SIF0OUT is tested
			P6OUT |= BIT4;		// if SIF0OUT=1 then a LED (P1.7) is switched on	
		else
			P6OUT &= ~BIT4;		// otherwise the LED is switched off
		
		if (SIFCTL3 & 0x02)		// SIF1OUT is tested
			P6OUT |= BIT5;		// if SIF1OUT=1 then a LED (P1.6) is switched on
		else
			P6OUT &= ~BIT5;		// otherwise the LED is switched off
#endif
	}
}

#pragma vector=SCANIF_VECTOR
__interrupt void ISR_ScanIF(void)
{
	if ((SIFCTL1 & SIFIFG3) && (SIFCTL1 & SIFIE3))
	{
		if ((SIFCNT == 0x07) && !calibrating)	// Example criteria for run-time calibration
		{
			LPM3_EXIT;					// Wake up the while loop in main() to start CalScanIF()
		}
		DisplayByte(SIFCNT);
		SIFCTL1 &= ~SIFIFG3;			// Clear interrupt flag
	}
	if ((SIFCTL1 & SIFIE1) && (SIFCTL1 & SIFIFG1))		// For initialization of ScanIF and run-time calibration
	{
		LPM3_EXIT;						// Wake up after sampling is done.
		SIFCTL1 &= ~SIFIFG1;			// Clear interrupt flag
	}
}
