// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted 
// by Texas Instruments is distributed as "freeware".  You may, 
// only under TI's copyright in the Program, use and modify the 
// Program without any charge or restriction.  You may 
// distribute to third parties, provided that you transfer a 
// copy of this license to the third party and the third party 
// agrees to these terms by its first use of the Program. You 
// must reproduce the copyright notice and any other legend of 
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains 
// copyrighted material, trade secrets and other TI proprietary 
// information and is protected by copyright laws, 
// international copyright treaties, and trade secret laws, as 
// well as other intellectual property laws.  To protect TI's 
// rights in the Program, you agree not to decompile, reverse 
// engineer, disassemble or otherwise translate any object code 
// versions of the Program to a human-readable form.  You agree 
// that in no event will you alter, remove or destroy any 
// copyright notice included in the Program.  TI reserves all 
// rights not specifically granted under this license. Except 
// as specifically provided herein, nothing in this agreement 
// shall be construed as conferring by implication, estoppel, 
// or otherwise, upon you, any license or other right under any 
// TI patents, copyrights or trade secrets.
//
//*******************************************************************************
// Set DAC values for 'FW LC sensors: Damped vs. Undamped
// Sensor wheel must be spinning during calibration.
//
// Zack Mak
// Texas Instruments
// January 2014
//*******************************************************************************

#include "msp430xw42x.h"
#define __msp430xW42x

const unsigned char PSM[] = {
	0x00,  // no rotation
	0x01,  // turns right
	0x0C,  // turns left (-1)
	0x49,  // error
	0x00,  // turns left
	0x01,  // no rotation
	0x48,  // error
	0x09,  // turns right
	0x02,  // turns right (+1)
	0x41,  // error
	0x08,  // no rotation
	0x09,  // turns left
	0x40,  // error
	0x01,  // turns left
	0x08,  // turns right
	0x09   // no rotation
};

#define HYSTERESIS			5
#define STATE_SEPERATION	20
#define INIT_CAL_STEP		0x80
#define RUNTIME_CAL_STEP	2

unsigned int	threshold_h1, threshold_h2,		// Upper threshold
				threshold_l1, threshold_l2;		// Lower threshold

void InitScanIF(void)
{
	int i;
	unsigned int	upper1 = 0,
					upper2 = 0,
					lower1 = 0xffff,
					lower2 = 0xffff,
					state1, state2,			// Measured output state
					tempDAC1, tempDAC2,		// Temp DAC value
					step1, step2;			// Calibration step size
	
	
	// Initialize Timing State Machine
	SIFTSM0 = 0x0000;
	
	SIFTSM1 = 0x0420;	// ACLK sync
	SIFTSM2 = 0x002C;	// DAC=off, CA=off,  1xSIFCLK, excitation	SIFCH.0
	SIFTSM3 = 0x0424;	// DAC=off, CA=off,  1xACLK					SIFCH.0
	SIFTSM4 = 0x6934;	// DAC=on,  CA=on,  14xSIFCLK				SIFCH.0
	SIFTSM5 = 0x2174;	// DAC=on,  CA=on,   5xSIFCLK, SIFRSON		SIFCH.0

	SIFTSM6 = 0x04A1;	// ACLK sync
	SIFTSM7 = 0x00AD;	// DAC=off, CA=off,  1xSIFCLK, excitation	SIFCH.1	SIFTESTS1
	SIFTSM8 = 0x04A5;	// DAC=off, CA=off,  1xACLK					SIFCH.1	SIFTESTS1
	SIFTSM9 = 0x69B5;	// DAC=on,  CA=on,  14xSIFCLK				SIFCH.1	SIFTESTS1
	SIFTSM10 = 0x21F5;	// DAC=on,  CA=on,   5xSIFCLK, SIFRSON		SIFCH.1	SIFTESTS1

	SIFTSM11 = 0x0200;	// stop

	// Initialize Scan Interface Control Registers
	
	// VCC/2, Excitation enabled, Test Channel SIFTCH1 = SIFCH1, SIFTCH0 = SIFCH0
	SIFCTL2 = 0x0150;
	for (i=0;i<=857;i++);				// wait loop. SIFVCCEx has to settle
	SIFCTL3 = 0x4000;					// SIF0OUT selected for S1, SIF1OUT selected for S2
										// (S1 and S2 are input signals of Processing State Machine)
	
	// initialize PSM counters (Sampling rate based on 32.768kHz ACLK)
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_162 + SIFDIV2_8;		// 25Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_162 + SIFDIV2_4;		// 50Hz sampling rate
	SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_162 + SIFDIV2_2;		// 101Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_162;					// 202Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_42 + SIFDIV2_2;		// 390Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_54;					// 607Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_42;					// 780Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_18 + SIFDIV2_2;		// 910Hz sampling rate
	// SIFCTL4 = SIFCNT1ENM + SIFCNT1ENP + SIFDIV3_30;						// 1092Hz sampling rate
	
	SIFCTL5 = 0x0045;					// SIFCLK is controlled by SIFTSMx
										// SIFCLK = 1MHz
	SIFPSMV = (unsigned int) &PSM;		// Definition of PSM vector

	SIFCTL1 |= SIFEN;					// switch on Scan Interface.
	
	//********* Calibration *****************
	// This calibration find the DAC value which distinguish the "threshold"
	// between measured state 0 and 1.
	// When the disc rotates, two "thresholds" are found.
	// Threshold 1: Metal covers the LC sensor
	// Threshold 2: No metal coves the LC sensor
	// Then average the 2 "thresholds" to get the final DAC values for metal
	// detection.
	do
	{	
		// Test 1
		// Initialize DAC digital control values
		SIFDACR0 = 0x0200;  // SIFCH.0 - set to VCC/2
		SIFDACR1 = 0x0200;
		SIFDACR2 = 0x0200;  // SIFCH.1 - set to VCC/2
		SIFDACR3 = 0x0200;
		tempDAC1  = 0x0200;
		tempDAC2  = 0x0200;
		
		step1 = INIT_CAL_STEP;
		step2 = INIT_CAL_STEP;
		
		state1 = 1;
		state2 = 1;
		
		// Start sampling the status of each sensor
		SIFCTL1 = SIFIE1 + SIFEN;
		
		// Find the "DAC threshold" between state 1 and state 0
		do
		{
			// Wait for sample
			SIFCTL1 &= ~SIFIFG1;
			LPM3;

			// 1. Set the DAC value to minimum. 0x0200 is low enough
			// 		If DAC value is low, it will get the measure state "1".
			// 2. Increase DAC value until measure state becomes "0"
			// 3. Reduce the calibration step size by half
			// 4. Decrease DAC value until measure state becomes "1"
			// 5. Reduce the calibration step size by half
			// 6. Repeat step 2 to 5 until the calibration step size becomes 0
			
			// For sensor 1
			if (step1)		// Skip if the threshold is already found
			{
				// Last state was 1
				if (state1)
				{
					state1 = SIFCTL3 & 0x01;	// Record the new state
					if (state1)					// New state is 1, keep going
					{
						tempDAC1 += step1;
						if (tempDAC1 > 0x03FF) tempDAC1 = 0x03FF;	// Prevent overflow
						
					}
					else						// New state is 0, change direction
					{
						step1 >>= 1;			// Reduce step size by half
						tempDAC1 -= step1;
					}
				}
				// Last state was 0
				else
				{
					state1 = SIFCTL3 & 0x01;	// Record the new state
					if (state1)					// New state is 1, change direction
					{
						step1 >>= 1;
						tempDAC1 += step1;
					}
					else						// New state is 0, keep going
					{
						tempDAC1 -= step1;
					}
				}
				
				// Update new DAC value
				SIFDACR0 = tempDAC1;
				SIFDACR1 = tempDAC1;
			}
			
			// For sensor 2
			if (step2)		// Skip if the threshold is already found
			{
				// Last state is 1
				if (state2)
				{
					state2 = SIFCTL3 & 0x02;	// Record the new state
					if (state2)					// New state is 1, keep going
					{
						tempDAC2 += step2;
						if (tempDAC2 > 0x03FF) tempDAC2 = 0x03FF;	// Prevent overflow
						
					}
					else						// New state is 0, change direction
					{
						step2 >>= 1;			// Reduce step size by half
						tempDAC2 -= step2;
					}
				}
				// Last state is 0
				else
				{
					state2 = SIFCTL3 & 0x02;	// Record the new state
					if (state2)					// New state is 1, change direction
					{
						step2 >>= 1;
						tempDAC2 += step2;
					}
					else						// New state is 0, keep going
					{
						tempDAC2 -= step2;
					}
				}
				
				// Update new DAC value
				SIFDACR2 = tempDAC2;
				SIFDACR3 = tempDAC2;
			}
		}while (step1 || step2);		// Wait until both sensors finish the process
		
		// Turn off SCANIF
		SIFCTL1 = 0;

		// This is to distinguish upper threshold and lower threshold
		if (tempDAC1 > upper1) upper1 = tempDAC1;
		if (tempDAC1 < lower1) lower1 = tempDAC1;

		if (tempDAC2 > upper2) upper2 = tempDAC2;
		if (tempDAC2 < lower2) lower2 = tempDAC2;

		__no_operation();
		
		// If the upper and lower thresholds have enough separation, that means
		// it is now able to detect the state of the rotating disc
		if (( (upper1 - lower1) > STATE_SEPERATION)
			&&	( (upper2 - lower2) > STATE_SEPERATION))
		{
			// Record the thresholds
			threshold_h1 = upper1;
			threshold_h2 = upper2;
			threshold_l1 = lower1;
			threshold_l2 = lower2;
			
			break;		// Finding threshold done. Exit the while loop.
		}
	}
	while (1);

	__no_operation();

	upper1 += lower1;
	upper1 >>= 1;
	SIFDACR0 = upper1 + HYSTERESIS;		// Calculate DAC values for sensor 1
	SIFDACR1 = upper1 - HYSTERESIS;

	upper2 += lower2;
	upper2 >>= 1;
	SIFDACR2 = upper2 + HYSTERESIS;		// Calculate DAC values for sensor 2
	SIFDACR3 = upper2 - HYSTERESIS;

//	SIFCTL4 = 0x33D4;					// reduce sampling rate to 50Hz
	SIFCTL1 = SIFIE3 + SIFEN;			// switch on Scan Interface. enable interrupt
}

// Run time calibration. 
void CalScanIF(void)
{
	unsigned int	upper1=0,
					upper2=0,
					lower1=0xffff,
					lower2=0xffff,
					cal_state1,
					cal_state2;		// State of calibration:
									// 0-(for threshold_h) initial state
									// 1-(for threshold_h) Threshold is higher than before, adjust DAC value upward
									// 2-(for threshold_h) Threshold is lower than before, adjust DAC value downward
									// 3-(for threshold_l) initial state
									// 4-(for threshold_l) Threshold is higher than before, adjust DAC value upward
									// 5-(for threshold_l) Threshold is lower than before, adjust DAC value downward
									// 6-finished
	do
	{
		cal_state1 = 0;
		cal_state2 = 0;
		
		// Initialize DAC digital control values
		SIFDACR6 = threshold_h1;
		SIFDACR7 = threshold_h2;
		
		do
		{
			// Insert Test Sample	
			SIFCTL1 |= SIFIE1 + SIFTESTD;
			do
			{
				SIFCTL1 &= ~SIFIFG1;
				LPM3;
			}while (SIFCTL1 & SIFTESTD);
			SIFCTL1 &= ~SIFIE1;
			
			// For sensor 1
			if (SIFCTL3 & SIF0OUT)							// Searching for higher threshold
			{
				if (SIFCTL2 & SIFTCH0OUT)
				{
					switch (cal_state1)
					{
						case 0:								// Initial state
							SIFDACR6 += RUNTIME_CAL_STEP;	// Threshold is higher than previous value
							cal_state1 = 1;					// DAC value should be adjusted upward
							break;
							
						case 1:								// Adjusting DAC value upward
							SIFDACR6 += RUNTIME_CAL_STEP;	// Threshold is still high
							break;
							
						case 2:								// Adjusting DAC value downward
							cal_state1 = 3;					// Threshold found, go for threshold_l
							upper1 = SIFDACR6;				// Record the current DAC value
							SIFDACR6 = threshold_l1;		// Prepare for searching lower threshold;
							break;
							
						default: break;
					}
				}
				else
				{
					switch (cal_state1)
					{
						case 0:								// Initial state
							SIFDACR6 -= RUNTIME_CAL_STEP;	// Threshold is lower than previous value
							cal_state1 = 2;					// DAC value should be adjusted downward
							break;
							
						case 1:								// Adjusting DAC value upward
							cal_state1 = 3;					// Threshold found, go for threshold_l
							upper1 = SIFDACR6;				// Record the current DAC value
							SIFDACR6 = threshold_l1;		// Prepare for searching lower threshold;
							break;
							
						case 2:								// Adjusting DAC value downward
							SIFDACR6 -= RUNTIME_CAL_STEP;	// Threshold is still low
							break;
							
						default: break;
					}
				}
			}
			else											// Searching for lower threshold
			{
				if (SIFCTL2 & SIFTCH0OUT)
				{
					switch (cal_state1)
					{
						case 3:								// Initial state
							SIFDACR6 += RUNTIME_CAL_STEP;	// Threshold is higher than previous value
							cal_state1 = 4;					// DAC value should be adjusted upward
							break;
							
						case 4:								// Adjusting DAC value upward
							SIFDACR6 += RUNTIME_CAL_STEP;	// Threshold is still high
							break;
							
						case 5:								// Adjusting DAC value downward
							cal_state1 = 6;					// Threshold found, go for threshold_l
							lower1 = SIFDACR6;				// Record the current DAC value
							break;
							
						default: break;
					}
				}
				else
				{
					switch (cal_state1)
					{
						case 3:								// Initial state
							SIFDACR6 -= RUNTIME_CAL_STEP;	// Threshold is lower than previous value
							cal_state1 = 5;					// DAC value should be adjusted downward
							break;
							
						case 4:								// Adjusting DAC value upward
							cal_state1 = 6;					// Threshold found, go for threshold_l
							lower1 = SIFDACR6;				// Record the current DAC value
							break;
							
						case 5:								// Adjusting DAC value downward
							SIFDACR6 -= RUNTIME_CAL_STEP;	// Threshold is still low
							break;
							
						default: break;
					}
				}
			
			}
			// For sensor 2
			if (SIFCTL3 & SIF1OUT)							// Searching for higher threshold
			{
				if (SIFCTL2 & SIFTCH1OUT)
				{
					switch (cal_state2)
					{
						case 0:								// Initial state
							SIFDACR7 += RUNTIME_CAL_STEP;	// Threshold is higher than previous value
							cal_state2 = 1;					// DAC value should be adjusted upward
							break;
							
						case 1:								// Adjusting DAC value upward
							SIFDACR7 += RUNTIME_CAL_STEP;	// Threshold is still high
							break;
							
						case 2:								// Adjusting DAC value downward
							cal_state2 = 3;					// Threshold found, go for threshold_l
							upper2 = SIFDACR7;				// Record the current DAC value
							SIFDACR7 = threshold_l2;		// Prepare for searching lower threshold;
							break;
							
						default: break;
					}
				}
				else
				{
					switch (cal_state2)
					{
						case 0:								// Initial state
							SIFDACR7 -= RUNTIME_CAL_STEP;	// Threshold is lower than previous value
							cal_state2 = 2;					// DAC value should be adjusted downward
							break;
							
						case 1:								// Adjusting DAC value upward
							cal_state2 = 3;					// Threshold found, go for threshold_l
							upper2 = SIFDACR7;				// Record the current DAC value
							SIFDACR7 = threshold_l2;		// Prepare for searching lower threshold;
							break;
							
						case 2:								// Adjusting DAC value downward
							SIFDACR7 -= RUNTIME_CAL_STEP;	// Threshold is still low
							break;
							
						default: break;
					}
				}
			}
			else											// Searching for lower threshold
			{
				if (SIFCTL2 & SIFTCH1OUT)
				{
					switch (cal_state2)
					{
						case 3:								// Initial state
							SIFDACR7 += RUNTIME_CAL_STEP;	// Threshold is higher than previous value
							cal_state2 = 4;					// DAC value should be adjusted upward
							break;
							
						case 4:								// Adjusting DAC value upward
							SIFDACR7 += RUNTIME_CAL_STEP;	// Threshold is still high
							break;
							
						case 5:								// Adjusting DAC value downward
							cal_state2 = 6;					// Threshold found, go for threshold_l
							lower2 = SIFDACR7;				// Record the current DAC value
							break;
							
						default: break;
					}
				}
				else
				{
					switch (cal_state2)
					{
						case 3:								// Initial state
							SIFDACR7 -= RUNTIME_CAL_STEP;	// Threshold is lower than previous value
							cal_state2 = 5;					// DAC value should be adjusted downward
							break;
							
						case 4:								// Adjusting DAC value upward
							cal_state2 = 6;					// Threshold found, go for threshold_l
							lower2 = SIFDACR7;				// Record the current DAC value
							break;
							
						case 5:								// Adjusting DAC value downward
							SIFDACR7 -= RUNTIME_CAL_STEP;	// Threshold is still low
							break;
							
						default: break;
					}
				}
			
			}
		}while ((cal_state1 != 6) || (cal_state2 != 6));	// Wait until both sensors finish this process
	}
	while (( (upper1 - lower1) < STATE_SEPERATION)
			||	( (upper2 - lower2) < STATE_SEPERATION));
		
	// Record the thresholds
	threshold_h1 = upper1;
	threshold_h2 = upper2;
	threshold_l1 = lower1;
	threshold_l2 = lower2;
	
	__no_operation();

	upper1 += lower1;
	upper1 >>= 1;
	SIFDACR0 = upper1 + HYSTERESIS;  // Calculation of average value
	SIFDACR1 = upper1 - HYSTERESIS;

	upper2 += lower2;
	upper2 >>= 1;
	SIFDACR2 = upper2 + HYSTERESIS;  // Calculation of average value
	SIFDACR3 = upper2 - HYSTERESIS;
}
